﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Node.Core2.Requestor;

namespace EN_NodeSampleCode
{
    public class SampleCode
    {
        #region Property for Proxy
        /// <summary>
        /// Set value to True to invoke Proxy Procedure, The default value is True.
        /// </summary>
        public bool IsProxy { get;set; }
        /// <summary>
        /// value for proxy url ie. http://192.168.88.56:546
        /// </summary>
        public string ProxyURL { get; set; }
        /// <summary>
        /// value for proxy user.
        /// </summary>
        public string ProxyUser { get; set; }
        /// <summary>
        /// value proxy user's password.
        /// </summary>
        public string ProxyPassword { get; set; }

        #endregion

        #region Property for Authentication (Required)
        /// <summary>
        /// http://localhost:3930/Node.WebServices/NodeServices2.asmx
        /// </summary>
        public string URL {get;set;}
        public string NAASUser { get; set; }
        public string NAASPassword { get; set; }
        /// <summary>
        /// default value is password
        /// </summary>
        public string AuthMethod { get; set; }
        /// <summary>
        /// default value is default
        /// </summary>
        public string Domain { get; set; }

        #endregion

        #region Property for Submit (Required)
        /// <summary>
        /// default value is EMTS
        /// </summary>
        public string DataFlow { get; set; }
        /// <summary>
        /// default value is EMTSSubmit
        /// </summary>
        public string FlowOperation { get; set; } 
        /// <summary>
        /// value True to send plain XML file, otherwise Zipped file.
        /// </summary>
        public bool IsXML { get; set; }
        /// <summary>
        /// default value is EMTS.xml
        /// </summary>
        public string DocName { get; set; }
        public byte[] Content { get; set; }
        #endregion

        #region Property for Query (Reruired)

        public string Request { get; set; }
        public DateTime StartDate { get; set; }
        public DateTime EndDate { get; set; }

        #endregion

        #region Property for Download (Reruired)

        public string TransationID { get; set; }

        #endregion

        #region Property for Output Result
        /// <summary>
        /// Return transaction id if calling web services is successful, otherwise error message.
        /// </summary>
        public string ResultMsg { get; set; }
        public byte[] ResultContent { get; set; }

        #endregion
        
        /// <summary>
        /// Construcator for SampleCode Which sets the default value for the properties.
        /// </summary>
        public SampleCode()
        {
            IsProxy = true;
            AuthMethod = "password";
            Domain = "default";

            IsXML = true;
            DataFlow = "EMTS";
            FlowOperation = "EMTSSubmit";
            DocName = "EMTS.xml";
        }
        /// <summary>
        /// Invoke process to call authentication and submit to ENNode
        /// </summary>
        public void InvokeSubmitWS()
        {
            string sToken = "";

            NodeRequestor req = new NodeRequestor(ProxyURL);

            //PROXY
            if (IsProxy)
            {
                System.Net.NetworkCredential cred = new System.Net.NetworkCredential(ProxyUser, ProxyPassword);
                System.Net.WebProxy proxy = new System.Net.WebProxy(ProxyURL, true, null, cred);
                req.Proxy = proxy;
            }
            //PROXY

            //Authentication 
            Authenticate auth = new Authenticate();
            auth.userId = NAASUser;
            auth.credential = NAASPassword;
            auth.authenticationMethod = AuthMethod;
            auth.domain = Domain;

            try
            {
                AuthenticateResponse resp = req.Authenticate(auth);
                sToken = resp.securityToken;
            }
            catch (Exception ex)
            {
                ResultMsg = "Error Authentication:" + ex.Message;
            }    
            //Authentication


            //Submit
            Node.Core2.Requestor.Submit submit = new Node.Core2.Requestor.Submit();
            submit.securityToken = sToken;
            submit.dataflow = DataFlow;
            submit.flowOperation = FlowOperation;
            submit.transactionId = "";

            NodeDocumentType[] retDocs = new NodeDocumentType[1];
            NodeDocumentType newDoc = new NodeDocumentType();

            if (!IsXML)
            {
                newDoc.documentFormat = Node.Core2.Requestor.DocumentFormatType.ZIP;
            }
            else
            {
                newDoc.documentFormat = Node.Core2.Requestor.DocumentFormatType.XML;
            }

            newDoc.documentName = DocName;
            newDoc.documentContent = new AttachmentType();
            newDoc.documentContent.Value = Content;
            retDocs[0] = newDoc;

            submit.documents = retDocs;

            try
            {
                Node.Core2.Requestor.StatusResponseType submitresp = req.Submit(submit);
            }
            catch (Exception ex)
            {
                ResultMsg = "Erro Submit:" + ex.Message;
            }

            ResultMsg = submit.transactionId;
            //Submit

        }
        /// <summary>
        /// Invoke process to call authentication and Query to ENNode
        /// </summary>
        public void InvokeQueryWS()
        {
            string sToken = "";

            NodeRequestor req = new NodeRequestor(ProxyURL);

            //PROXY
            if (IsProxy)
            {
                System.Net.NetworkCredential cred = new System.Net.NetworkCredential(ProxyUser, ProxyPassword);
                System.Net.WebProxy proxy = new System.Net.WebProxy(ProxyURL, true, null, cred);
                req.Proxy = proxy;
            }
            //PROXY

            //Authentication 
            Authenticate auth = new Authenticate();
            auth.userId = NAASUser;
            auth.credential = NAASPassword;
            auth.authenticationMethod = AuthMethod;
            auth.domain = Domain;

            try
            {
                AuthenticateResponse resp = req.Authenticate(auth);
                sToken = resp.securityToken;
            }
            catch (Exception ex)
            {
                ResultMsg = "Error Authentication:" + ex.Message;
            }
            //Authentication

            //Query

            Node.Core2.Requestor.Query qry = new Node.Core2.Requestor.Query();
            qry.dataflow = DataFlow;
            qry.request = Request;
            qry.securityToken = sToken;
            qry.rowId = "0";
            qry.maxRows = "-1";

            Node.Core2.Requestor.ParameterType[] parameters = new Node.Core2.Requestor.ParameterType[3];

            parameters[0] = new Node.Core2.Requestor.ParameterType();
            parameters[0].parameterName = "StartDate";
            parameters[0].Value = StartDate.ToShortDateString();

            parameters[0] = new Node.Core2.Requestor.ParameterType();
            parameters[0].parameterName = "EndDate";
            parameters[0].Value = EndDate.ToShortDateString();

            parameters[0] = new Node.Core2.Requestor.ParameterType();
            parameters[0].parameterName = "DocumentName";
            parameters[0].Value = DocName;
            qry.parameters = parameters;

            try
            {
                Node.Core2.Requestor.ResultSetType result = req.Query(qry);
                ResultMsg = result.results.Any[0].OuterXml;
            }
            catch (Exception ex)
            {
                ResultMsg = "Erro Query:" + ex.Message;
            }

            //Query

        }
        /// <summary>
        /// Invoke process to call authentication and Download from ENNode
        /// </summary>
        public void InvokeDownloadWS()
        {
            string sToken = "";

            NodeRequestor req = new NodeRequestor(ProxyURL);

            //PROXY
            if (IsProxy)
            {
                System.Net.NetworkCredential cred = new System.Net.NetworkCredential(ProxyUser, ProxyPassword);
                System.Net.WebProxy proxy = new System.Net.WebProxy(ProxyURL, true, null, cred);
                req.Proxy = proxy;
            }
            //PROXY

            //Authentication 
            Authenticate auth = new Authenticate();
            auth.userId = NAASUser;
            auth.credential = NAASPassword;
            auth.authenticationMethod = AuthMethod;
            auth.domain = Domain;

            try
            {
                AuthenticateResponse resp = req.Authenticate(auth);
                sToken = resp.securityToken;
            }
            catch (Exception ex)
            {
                ResultMsg = "Error Authentication:" + ex.Message;
            }
            //Authentication

            //DownlLoad

            Node.Core2.Requestor.Download download = new Node.Core2.Requestor.Download();
            download.transactionId = "" + TransationID;
            download.securityToken = sToken;
            Node.Core2.Requestor.NodeDocumentType[] retDocs = new Node.Core2.Requestor.NodeDocumentType[0];
            download.documents = retDocs;
            download.dataflow = DataFlow;
            try
            {
                retDocs = req.Download(download);
                
                foreach (Node.Core2.Requestor.NodeDocumentType doc in retDocs)
                {
                    ResultContent = doc.documentContent.Value;
                }

            }
            catch (Exception ex )
            {
                ResultMsg = "Erro Download:" + ex.Message;
            }

        }
    }
}
